/*============================================================================
  File:     FixUsingPageRestore.sql

  Summary:  This script repairs the DemoPageRestore
			database using single page restore.

  Date:     June 2009

  SQL Server Versions:
		10.0.2531.00 (SS2008 SP1)
		9.00.4035.00 (SS2005 SP3)
------------------------------------------------------------------------------
  Written by Paul S. Randal, SQLskills.com

  (c) 2011, SQLskills.com. All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you include this copyright and give due
  credit, but you must obtain prior permission before blogging this code.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

-- Disaster recovery simulation of a zero'd page, using
-- single-page restore.

-- Must execute Setup scripts first for this to work.

-- Innocent user query on the sales table
SELECT SUM (salesAmount) FROM
	DemoPageRestore.dbo.sales
WHERE salesAmount < $1.00;
GO

-- Uh-oh - corruption!
-- Check the database using DBCC CHECKDB
DBCC CHECKDB (DemoPageRestore)
	WITH ALL_ERRORMSGS, NO_INFOMSGS;
GO

-- What does the page look like? Try doing an
-- interpreted dump of the page using DBCC PAGE
DBCC TRACEON (3604); -- necessary for correct output
GO
DBCC PAGE (DemoPageRestore, 1, 158, 3);
GO

-- DBCC PAGE can't work out how to dump the page
-- because the header is zero'd. Try a hex dump.
DBCC PAGE (DemoPageRestore, 1, 158, 2);
GO

-- Wow - totally zero! Looks like an IO error.

-- First thing to do, disallow user access.
-- Remember not to set it to OFFLINE otherwise not
-- even sysadmin can access the database.
ALTER DATABASE DemoPageRestore SET RESTRICTED_USER;
GO

-- Before we decide to fix the database using single-page
-- restore, let's see if any other pages are corrupt
SELECT * FROM msdb..suspect_pages;
GO

-- Now restore the single damaged page
USE master;
GO

-- First backup the tail of the log...
BACKUP LOG DemoPageRestore TO
	DISK = 'C:\SQLskills\DemoPageRestore_LOG_TAIL.bak'
	WITH INIT;
GO

-- And then start restoring, first with the full backup...
RESTORE DATABASE DemoPageRestore
	PAGE = '1:158' FROM
	DISK = 'C:\SQLskills\DemoPageRestore.bak';
GO

-- Now the first log backup...
RESTORE LOG DemoPageRestore FROM
	DISK = 'C:\SQLskills\DemoPageRestore_LOG1.bak'
	WITH NORECOVERY;
GO
-- And so on...

-- Finally with the tail of the log
RESTORE LOG DemoPageRestore FROM
	DISK = 'C:\SQLskills\DemoPageRestore_LOG_TAIL.bak'
	WITH NORECOVERY;
GO

-- And finish up with recovery. Notice all restore commands
-- used NORECOVERY to prevent mistakes
RESTORE DATABASE DemoPageRestore WITH RECOVERY;
GO

-- Check the database for corruption again
DBCC CHECKDB (DemoPageRestore)
	WITH ALL_ERRORMSGS, NO_INFOMSGS;
GO

-- Bring the database online again
ALTER DATABASE DemoPageRestore SET MULTI_USER;
GO

-- Final sanity check - run the user query again
SELECT SUM (salesAmount) FROM DemoPageRestore.dbo.sales
	WHERE salesAmount < $1.00;
GO
